#region CVS Version Header
/*
 * $Id: RequestParameter.cs,v 1.1 2004/08/20 18:27:10 t_rendelmann Exp $
 * Last modified by $Author: t_rendelmann $
 * Last modified at $Date: 2004/08/20 18:27:10 $
 * $Revision: 1.1 $
 */
#endregion

using System;
using System.IO;
using System.Net;

namespace NewsComponents.Net
{
	/// <summary>
	/// Called on every web request, that was queued.
	/// </summary>
	public delegate void RequestQueuedCallback(Uri requestUri, int priority);
	/// <summary>
	/// Called on every queued web request, that is now about to really make the request to 
	/// destination server.
	/// </summary>
	public delegate void RequestStartCallback(Uri requestUri, ref bool cancel);
	/// <summary>
	/// Called, if the web request caused an exception, that is not yet handled by the class itself.
	/// </summary>
	public delegate void RequestExceptionCallback(Uri requestUri, Exception e);
	/// <summary>
	/// Called on every queued request, when the real fetch is finished.
	/// </summary>
	public delegate void RequestCompleteCallback(Uri requestUri, Stream response, Uri newUri, string eTag, DateTime lastModified, RequestResult result);

	/// <summary>
	/// State of a successfully made web request.
	/// </summary>
	public enum RequestResult {
		/// <summary>
		/// Request returns a new response stream.
		/// </summary>
		OK,
		/// <summary>
		/// Web request results in a not modfied response.
		/// </summary>
		NotModified
	}

	/// <summary>
	/// Class is a container for a all the needed web request parameter.
	/// </summary>
	public class RequestParameter {
		/// <summary>
		/// Constructor initialize a RequestParameter instance.
		/// </summary>
		/// <param name="address">Requested Web Url</param>
		/// <param name="userAgent">User Agent string the request should send</param>
		/// <param name="proxy">IWebProxy instance, if a proxy should be used</param>
		/// <param name="credentials">ICredentials instance, if credentials have to be used</param>
		/// <param name="ifModifiedSince">Advanced Request Header info</param>
		/// <param name="eTag">Advanced Request Header info</param>
		public RequestParameter(Uri address, string userAgent, 
			IWebProxy proxy, ICredentials credentials, DateTime ifModifiedSince, string eTag) {
			
			this.requestUri = address;
			this.userAgent = userAgent;
			this.proxy = proxy;
			this.credentials = credentials;
			this.lastModified = ifModifiedSince;
			this.eTag = eTag;
		}

		private Uri requestUri;
		/// <summary>
		/// Gets the Request Uri.
		/// </summary>
		public Uri RequestUri {	get { return this.requestUri; } }

		private string userAgent;
		/// <summary>
		/// Gets the Request user agent string
		/// </summary>
		public string UserAgent { get { return this.userAgent; }	}

		private IWebProxy proxy;
		/// <summary>
		/// Gets the proxy.
		/// </summary>
		public IWebProxy Proxy { get { return this.proxy; }	}

		private ICredentials credentials;
		/// <summary>
		/// Gets the credentials.
		/// </summary>
		public ICredentials Credentials { get { return this.credentials; }	}

		private DateTime lastModified;
		/// <summary>
		/// Gets the "last modified since" date
		/// </summary>
		public DateTime LastModified { 
			get { return this.lastModified; }	
			set { this.lastModified = value; }	
		}

		private string eTag;
		/// <summary>
		/// Gets ETag header info
		/// </summary>
		public string ETag { 
			get { return this.eTag; }	
			set { this.eTag = value; }	
		}
	}
}
