#region CVS Version Header
/*
 * $Id: HtmlHelper.cs,v 1.2 2005/01/18 17:36:44 carnage4life Exp $
 * Last modified by $Author: carnage4life $
 * Last modified at $Date: 2005/01/18 17:36:44 $
 * $Revision: 1.2 $
 */
#endregion

using System;
using System.Collections;
using System.IO; 
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Xml; 
using System.Diagnostics;

using NewsComponents.Collections;

namespace NewsComponents.Utils
{
	/// <summary>
	/// Helper class to work on HTML content.
	/// </summary>
	public sealed class HtmlHelper {
		
		private static Regex RegExFindHref = new Regex(@"<a\s+([^>]+\s+)?href\s*=\s*(?:""(?<1>[/\a-z0-9_][^""]*)""|'(?<1>[/\a-z0-9_][^']*)'|(?<1>[/\a-z0-9_]\S*))(\s[^>]*)?>(?<2>.*?)</a>", RegexOptions.Singleline | RegexOptions.IgnoreCase | RegexOptions.Compiled);
		private static Regex RegExBadTags = new Regex(@"<(?:script|object|meta|embed|frameset|i?frame|style|link)[\s>]", RegexOptions.IgnoreCase | RegexOptions.Compiled);
		private static Regex RegExAnyTags = new Regex("</?[^<>]+>", RegexOptions.Compiled); 

		/// <summary>
		/// Converts a relative url to an absolute one. baseUrl is used as the base to fix the other.
		/// </summary>
		/// <param name="url">Url to fix</param>
		/// <param name="baseUrl">base Url to be used</param>
		/// <returns></returns>
		public static string ConvertToAbsoluteUrl(string url, string baseUrl) {
			
			// we try to prevent the exception caused in the case the url is relative
			// (no scheme info) just for speed
			if (url.IndexOf(Uri.SchemeDelimiter) < 0 && baseUrl != null) {
				try {
					Uri baseUri= new Uri(baseUrl);
					return (new Uri(baseUri,url).ToString()); 
				} catch {}
			} 
			
			try{ 
				Uri uri = new Uri(url); 
				return uri.ToString(); 
			}catch(Exception){

				if (baseUrl != null) {
					try {
						Uri baseUri= new Uri(baseUrl);
						return (new Uri(baseUri,url).ToString()); 
					} catch (Exception) {
						return url;
					}
				} else {
					return url;
				}
			}
		}
		
		/// <summary>
		/// Returns a RelationHRefDictionary with links found in HTML &lt;a href=""> attributes.
		/// </summary>
		/// <param name="html">String to work on</param>
		/// <param name="baseUrl">An absolute Url to be used to fix relative links</param>
		/// <returns>RelationHRefDictionary with string Urls as key(s), and link text as items</returns>
		public static RelationHRefDictionary RetrieveLinks(string html, string baseUrl) {
			RelationHRefDictionary tbl = new RelationHRefDictionary();

			if (html == null)
				return tbl;

			for (Match m = RegExFindHref.Match(html); m.Success; m = m.NextMatch()) {
				string href =  ConvertToAbsoluteUrl(m.Groups[1].ToString(), baseUrl);
				href = RelationCosmos.RelationCosmos.UrlTable.Add(href);

				string linkText = m.Groups[2].ToString();	
				
				if (!tbl.ContainsKey(href))
					//tbl.Add(href, linkText);
					tbl.Add(href, new RelationHRefEntry(href, linkText, m.Index));
			}

			return tbl; 
		}


		/// <summary>
		/// Removes any tag of the form &lt;xxx> or &lt;/xxx>.
		/// </summary>
		/// <param name="html">string to work on</param>
		/// <returns>A Text-Only version of the provided input.</returns>
		public static string StripAnyTags(string html) {
			if (html == null)
				return String.Empty;
			return RegExAnyTags.Replace(html, String.Empty); 
		}

		/// <summary>
		/// Replace any bad tag of the form &lt;object> by &lt;bject> to make
		/// it does not work on rendering later
		/// </summary>
		/// <param name="html">string to work on</param>
		/// <returns>A cleaner version of the provided input.</returns>
		public static string StripBadTags(string html) {
			if (html == null)
				return String.Empty;
			if (RegExBadTags.IsMatch(html)) {
				html = RegExBadTags.Replace(html, new MatchEvaluator(HtmlHelper.RegexTagEvaluate));
			}
			return html;
		}

		private static string RegexTagEvaluate(Match m) {
			return string.Concat("<", m.ToString().Substring(1));
		}

		private HtmlHelper() {}

	}

}
