#region CVS Version Header
/*
 * $Id: ThreadedListViewColumnHeaderCollection.cs,v 1.7 2005/01/29 16:14:58 t_rendelmann Exp $
 * Last modified by $Author: t_rendelmann $
 * Last modified at $Date: 2005/01/29 16:14:58 $
 * $Revision: 1.7 $
 */
#endregion

using System;
using System.Collections;
using System.Windows.Forms;

namespace System.Windows.Forms.ThListView
{
	/// <summary>
	/// Summary description for ThreadedListViewColumnHeaderCollection.
	/// </summary>
	public class ThreadedListViewColumnHeaderCollection: System.Windows.Forms.ListView.ColumnHeaderCollection
	{
		public ThreadedListViewColumnHeaderCollection(ThreadedListView owner): base(((ListView)owner)) { 	} 

		#region Add
		public int Add(ThreadedListViewColumnHeader colHeader) {
			return base.Add(colHeader);
		}

		public new ThreadedListViewColumnHeader Add(string caption, int width, HorizontalAlignment textAlign) {
			return Add(null, caption, typeof(string), width, textAlign);
		}

		public ThreadedListViewColumnHeader Add(string id, string caption, Type valueType, int width, HorizontalAlignment textAlign) {
			ThreadedListViewColumnHeader c = new ThreadedListViewColumnHeader(id, valueType);
			c.Text = caption;
			c.Width = width;
			c.TextAlign = textAlign;
			int index = base.Add(c);
			return c;
		}
		#endregion

		public void AddRange(ThreadedListViewColumnHeader[] columns) {
			base.AddRange(columns);
		}

		public void Remove(string columnID) {
			ThreadedListViewColumnHeader c = this[columnID];
			base.Remove(c);
		}

		public new ThreadedListViewColumnHeader this[int displayIndex] { 
			get { 
				return ((ThreadedListViewColumnHeader)base[displayIndex]); 
			} 
		} 

		public ThreadedListViewColumnHeader this[string columnID] { 
			get { 
				if (columnID == null)
					throw new ArgumentNullException("columnID");
				int idx = this.GetIndexByKey(columnID);
				if (idx < 0)
					throw new InvalidOperationException("No ThreadedListViewColumnHeader found with ID '" + columnID + "'");
				return this[idx];
			} 
		}
 
		/// <summary>
		/// Returns the index of the Column identified by column ID.
		/// </summary>
		/// <param name="columnID">string</param>
		/// <returns>int. -1, if not found</returns>
		public int GetIndexByKey(string columnID) {
			for (int i=0; i < base.Count; i++) {
				ThreadedListViewColumnHeader c = this[i]; 
				if (c.Key == columnID)
					return i;
			}
			return -1;
		}

		/// <summary>
		/// Returns a collection for faster column key to index lookup.
		/// Can be used to create new items/subitems according to the current column keys.
		/// </summary>
		public ColumnKeyIndexMap GetColumnIndexMap() {
			ColumnKeyIndexMap map = new ColumnKeyIndexMap(base.Count);
			lock (this) {
				for (int i=0; i < base.Count; i++) {
					ThreadedListViewColumnHeader c = this[i]; 
					map.Add(c.Key, i);
				}
			}
			return map;
		}
	}

	#region helper classes
	/// <summary>
	/// Helper class for faster column key to index lookup.
	/// Can be used to create new items/subitems according to the current column keys.
	/// </summary>
	public class ColumnKeyIndexMap: Hashtable {
		public ColumnKeyIndexMap(): base() {}
		public ColumnKeyIndexMap(int capacity): base(capacity) {}
		public ColumnKeyIndexMap(IDictionary d): base(d) {}

		/// <summary>
		/// Set/Get the index of a column key
		/// </summary>
		public int this[string key] {
			get {
				return (int)base[key];
			}
			set {
				base[key] = (int)value;
			}
		}

		public void Add(string key, int value) {
			base.Add (key, value);
		}
		public bool Contains(string key) {
			return base.Contains (key);
		}
		public bool ContainsKey(string key) {
			return base.ContainsKey (key);
		}
		public bool ContainsValue(int value) {
			return base.ContainsValue (value);
		}
	}
	#endregion

}
