#region CVS Version Header
/*
 * $Id: PostReplyThreadHandler.cs,v 1.8 2004/11/01 07:21:22 t_rendelmann Exp $
 * Last modified by $Author: t_rendelmann $
 * Last modified at $Date: 2004/11/01 07:21:22 $
 * $Revision: 1.8 $
 */
#endregion

using System;
using System.Threading;
using System.IO;
using System.Net;
using System.Diagnostics;
using System.Windows.Forms;

using Logger = RssBandit.Common.Logging;
using NewsComponents;
using NewsComponents.Feed;

namespace RssBandit.WinGui {
	/// <summary>
	/// Summary description for PostReplyThreadHandler.
	/// </summary>
	public class PostReplyThreadHandler {
		
		private PostReplyThreadHandler() {	}
		public PostReplyThreadHandler(NewsHandler feedHandler, string commentApiUri, NewsItem item2post, NewsItem inReply2item) {
			this.feedHandler = feedHandler;
			this.commentApiUri = commentApiUri;
			this.item2post = item2post;
			this.inReply2item = inReply2item;
		}

		private static readonly log4net.ILog _log = Logger.Log.GetLogger(typeof(PostReplyThreadHandler));
		private AutoResetEvent workDone = new AutoResetEvent(false);
		private NewsHandler feedHandler = null;
		private string commentApiUri = null;
		private NewsItem item2post = null, inReply2item = null;
		private bool operationSucceeds = false;
		private Exception operationException = null;

		public string CommentApiUri {
			get {	return this.commentApiUri;	}
			set {	this.commentApiUri = value;	}
		}

		public NewsItem ItemToPost {
			get {	return this.item2post;	}
			set {	this.item2post = value;	}
		}

		public bool OperationSucceeds {
			get {	return this.operationSucceeds;	}
		}

		public Exception OperationException {
			get {	return this.operationException;	}
		}

		public AutoResetEvent WorkDone {
			get {	return workDone;	}
		}

		public void Run() {
			
			operationSucceeds = false;

			try {				
				this.feedHandler.PostCommentViaCommentAPI(commentApiUri, item2post, inReply2item); 
				operationSucceeds = true;
			} 
			catch(WebException we) {

				//Open file for writing 
				System.Text.StringBuilder sb = new System.Text.StringBuilder(); 
				StringWriter writeStream = null; 
				StreamReader readStream = null; 

				if(we.Response != null) { 
					writeStream = new StringWriter(sb); 

					//Retrieve input stream from response and specify encoding 
					Stream receiveStream     = we.Response.GetResponseStream();
					System.Text.Encoding encode = System.Text.Encoding.GetEncoding("utf-8");

					// Pipes the stream to a higher level stream reader with the required encoding format. 
					readStream = new StreamReader( receiveStream, encode );

					Char[] read = new Char[256]; 
    
					// Reads 256 characters at a time.    
					int count = readStream.Read( read, 0, 256 );


					while (count > 0) {
      
						// Dumps the 256 characters on a string and displays the string to the console.
						writeStream.Write(read, 0, count);
						count = readStream.Read(read, 0, 256);
      
					}   
					operationException = new WebException(sb.ToString(), we, we.Status, we.Response);

					if(writeStream != null){ writeStream.Close(); }
					if (readStream != null) { readStream.Close(); }

				} else {

					operationException = we;

				}
			}
			catch (Exception ex) {
				operationException = ex;
			}
			finally {
				workDone.Set();
			}
		}

	}
}